<?PHP
/**
 * Controls automatic translation of times.
 *
 * @author Joshua Ramon Enslin <joshua@museum-digital.de>
 */
declare(strict_types = 1);
require_once __DIR__ . '/inc/datesByCountry.php';

/**
 * Autotranslater class for times.
 */
final class NodaTimeAutotranslater {

    // TODO: Move these to NodaTimeAutotranslaterLocales

    const LANGS_SINCE_START_FORMAT_YEAR = [
        'ar' => '%s-',
        'de' => 'Seit %s',
        'en' => 'Since %s',
        'es' => 'Desde %s',
        'fa' => '%s-',
        'fr' => 'Depuis %s',
        'hu' => '%s-től',
        'id' => 'Sejak %s',
        'it' => 'Dal %s',
        'ka' => '%s წლიდან',
        'ko' => '%s부터',
        'pl' => 'Od %s r.',
        'pt' => '%s dC',
        'ro' => 'Din %s',
        'ru' => 'С %s г.',
        'ta' => '%s  முதல்',
        'tl' => 'Mula noong %s',
        'tr' => '%s-',

        // Languages that don't really need a specific locale
        'ja' => '%s以来',
        'zh' => '自%s以來',
    ];

    const LANGS_AFTER_START_FORMAT_YEAR = [
        'ar' => '%s-',
        'de' => 'Nach %s',
        'en' => 'After %s',
        'es' => 'Despues de %s',
        'fa' => '%s-',
        'fr' => 'Après %s',
        'hu' => '%s után',
        'id' => 'Setelah %s',
        'it' => 'Dopo il %s',
        'ka' => '%s წლის შემდეგ',
        'ko' => '%s 년 이후',
        'pl' => 'Po %s r.',
        'pt' => 'Depois de %s',
        'ro' => 'După %s',
        'ru' => 'после %s г.',
        'ta' => '%s  க்குப் பிறகு',
        'tl' => 'Pagkaraan ng %s',
        'tr' => '%s-',

        // Languages that don't really need a specific locale
        'ja' => '%s以降',
        'zh' => '%s以后',
    ];

    const LANGS_UNTIL_START_FORMAT_YEAR = [
        'ar' => '-%s',
        'de' => 'Bis %s',
        'en' => 'Until %s',
        'es' => 'Hasta %s',
        'fa' => '-%s',
        'fr' => 'Jusqu\'en %s',
        'hu' => '%s-ig',
        'id' => 'Sampai %s',
        'it' => 'Fino al %s',
        'ka' => '%s წლამდე',
        'ko' => '%s까지',
        'pl' => 'do %s roku',
        'pt' => 'até %s',
        'ro' => 'până în %s',
        'ru' => 'до %s г.',
        'ta' => '%s வரை',
        'tl' => 'Hanggang %s',
        'tr' => '%s-',

        // Languages that don't really need a specific locale
        'ja' => '%sまで',
        'zh' => '直到%s',
    ];

    const LANGS_BEFORE_START_FORMAT_YEAR = [
        'ar' => '-%s',
        'de' => 'Vor %s',
        'en' => 'Before %s',
        'es' => 'Antes de %s',
        'fa' => '-%s',
        'fr' => 'Avant %s',
        'hu' => '%s előtt',
        'id' => 'Sebelum %s',
        'it' => 'Prima del %s',
        'ka' => '%s წლამდე',
        'ko' => '%s 년 이전',
        'pl' => 'Przed %s',
        'pt' => 'Antes de %s',
        'ro' => 'înainte de %s',
        'ru' => 'до %s г.',
        'ta' => '%s க்கு முன்',
        'tl' => 'Bago ang %s',
        'tr' => '%s-',

        // Languages that don't really need a specific locale
        'ja' => '%s以前',
        'zh' => '%s之前',
    ];

    const LANGS_CENTURY_FORMAT = [
        'ar' => 'القرن ال %s',
        'de' => '%s. Jahrhundert',
        'en' => '%s. century',
        'es' => 'Siglo %s',
        'fa' => 'قرن %s',
        'fr' => '%sème siècle',
        'hu' => '%s. század',
        'id' => 'Abad ke-%s',
        'it' => '%sesimo secolo',
        'ka' => 'მე -%s საუკუნე',
        'ko' => '%s 세기',
        'pl' => '%s wiek',
        'pt' => 'Século %s',
        'ro' => 'Secolul al %s-lea',
        'ru' => '%s век',
        'ta' => '%s ஆம் நூற்றாண்டு',
        'tl' => 'Ika-%s na siglo',
        'tr' => '%s. yüzyıl',
        'ja' => '%s世紀',
        'zh' => '%s世紀',
    ];

    const LANGS_CENTURIES_FORMAT = [
        'ar' => 'القرن ال %s-%s',
        'de' => '%s.-%s. Jahrhundert',
        'en' => '%s.-%s. century',
        'es' => 'Siglo %s-%s',
        'fa' => 'قرن %s-%s',
        'fr' => '%s-%sème siècle',
        'hu' => '%s.-%s. század',
        'id' => 'Abad ke-%s-%s',
        'it' => '%s-%sesimo secolo',
        'ka' => 'მე -%s-%s საუკუნე',
        'ko' => '%s-%s 세기',
        'pl' => '%s-%s wiek',
        'pt' => 'Século %s-%s',
        'ro' => 'Secolul al %s-%s-lea',
        'ru' => '%s-%s век',
        'ta' => '%s-%s ஆம் நூற்றாண்டு',
        'tl' => 'Ika-%s hanggang ika-%s na siglo',
        'tr' => '%s.-%s. yüzyıl',
        'ja' => '%s世紀-%s世紀',
        'zh' => '%s-%s世紀',
    ];

    const LANGS_DECADE_FORMAT = [
        'ar' => '%s-%s',
        'de' => '%ser Jahre',
        'en' => '%ss',
        'es' => '%s-%s',
        'fa' => 'دهه %s',
        'fr' => 'Années %s',
        'hu' => '%s-as évek',
        'id' => 'Tahun %s-an',
        'it' => '%ss',
        'ka' => '%s-იანი წლები',
        'ko' => '%s 년대',
        'pl' => '%s roku',
        'pt' => 'Década de %s',
        'ro' => 'Anii %s',
        'ru' => '%s-е годы',
        'ta' => '%s கள்',
        'tl' => '%ss',
        'tr' => '%s\'ler',
        'ja' => '%s年代',
        'zh' => '%s年代',
    ];

    const LANGS_DECADES_FORMAT = [
        'ar' => '%s-%s',
        'de' => '%s-%ser Jahre',
        'en' => '%s-%ss',
        'es' => '%s-%s',
        'fa' => 'دهه %s-%s',
        'fr' => 'Années %s-%s',
        'hu' => '%s-%s-as évek',
        'id' => 'Tahun %s-an sampai tahun %s-an',
        'it' => '%s-%ss',
        'ka' => '%s-%s-იანი წლები',
        'ko' => '%s-%s 년대',
        'pl' => '%s-%s roku',
        'pt' => 'Décadas de %s-%s',
        'ro' => 'Anii %s-%s',
        'ru' => '%s-%s-е годы',
        'ta' => '%s-%s கள்',
        'tl' => '%s-%ss',
        'tr' => '%s-%s\'ler',
        'ja' => '%s年代から%s年代',
        'zh' => '%s年代-%s年代',
    ];

    const LANGS_SYLLABLE_CLEANING = [
        "hu" => [
            "10-as évek" => "10-es évek",
            "40-as évek" => "40-es évek",
            "50-as évek" => "50-es évek",
            "70-as évek" => "70-es évek",
            "90-as évek" => "90-es évek",
        ],
    ];

    /** @var MDMysqli */
    private MDMysqli $_mysqli_noda;
    /** @var integer */
    private int $_znum;
    /** @var MDMysqliStmt */
    private MDMysqliStmt $_insertStmt;

    /**
     * Formats a time according to a specified format.
     *
     * @return string
     */

    /**
     * Checks if a time is translatable.
     * Translatable times have either a counting time day and month or at least a month.
     *
     * @param string $zeit_beginn          Beginn year.
     * @param string $zeit_ende            End year.
     * @param string $zeit_zaehlzeit_monat Counting time month.
     *
     * @return NodaTimeAutotranslaterStatus
     */
    public static function check_translatability(string $zeit_beginn, string $zeit_ende, string $zeit_zaehlzeit_monat):NodaTimeAutotranslaterStatus {

        if ($zeit_ende === "?") {
            return NodaTimeAutotranslaterStatus::SINCE_START;
        }
        if ($zeit_beginn === "?") {
            return NodaTimeAutotranslaterStatus::UNTIL_END;
        }

        if (\intval($zeit_ende) >= 0 && \intval($zeit_beginn) < 0) {
            return NodaTimeAutotranslaterStatus::NOT;
        }

        if (($zeit_ende > 0 && \intval($zeit_ende) % 100 === 0 and (\intval($zeit_beginn) - 1) % 100 === 0)
            || (\intval($zeit_beginn) < 0 && \intval($zeit_beginn) % 100 === 0 and (\intval($zeit_ende) + 1) % 100 === 0)
        ) {
            return NodaTimeAutotranslaterStatus::CENTURY;
        }

        if ((\intval($zeit_ende) + 1) % 10 === 0 and \intval($zeit_beginn) % 10 === 0 and \intval($zeit_beginn) > 1000) {
            return NodaTimeAutotranslaterStatus::DECADE;
        }

        if (\intval($zeit_ende) < 0 && \intval($zeit_beginn) < 0
            || (\intval($zeit_ende) < 1000 and \trim($zeit_zaehlzeit_monat, ", .0") === "")
        ) {
            return NodaTimeAutotranslaterStatus::AS_YEAR_WITH_SUFFIX;
        }
        if ($zeit_ende === $zeit_beginn and \trim($zeit_zaehlzeit_monat, ", .0") === "") {
            return NodaTimeAutotranslaterStatus::ONLY_YEAR;
        }
        if ($zeit_ende !== $zeit_beginn and \trim($zeit_zaehlzeit_monat, ", .0") === "") {
            return NodaTimeAutotranslaterStatus::TIMESPAN_YEARS;
        }
        // Conditions speaking against translatability.
        if (\trim($zeit_zaehlzeit_monat, ", .0") === "") {
            return NodaTimeAutotranslaterStatus::NOT;
        }

        return NodaTimeAutotranslaterStatus::AS_MONTH_DAY;

    }

    /**
     * Gets suffix mode for years with suffix.
     *
     * @param integer $start Start year.
     * @param integer $end   End year.
     *
     * @return NodaTimeAutotranslaterSuffixMode
     */
    public static function getSuffixModeForYearsWSuffix(int $start, int $end):NodaTimeAutotranslaterSuffixMode {

        if ($start < 0 && $end < 0) {
            $suffixMode = NodaTimeAutotranslaterSuffixMode::BCE; // 2; // " v. Chr.";
        }
        else if ($end < 1000) {
            $suffixMode = NodaTimeAutotranslaterSuffixMode::CE; // " n. Chr.";
        }
        else $suffixMode = NodaTimeAutotranslaterSuffixMode::ANY; // Times larger than 1000 n. Chr.

        return $suffixMode;

    }

    /**
     * Applies suffix format to a time string.
     *
     * @param string                           $tLang      Two digit ANSI language code.
     * @param string                           $timeName   Time name.
     * @param NodaTimeAutotranslaterSuffixMode $suffixMode Suffix mode.
     *
     * @return string
     */
    public static function applyBcBceFormat(string $tLang, string $timeName, NodaTimeAutotranslaterSuffixMode $suffixMode):string {

        switch ($suffixMode) {
        case NodaTimeAutotranslaterSuffixMode::ANY:
            return $timeName;
        case NodaTimeAutotranslaterSuffixMode::CE:
            return \sprintf(NodaTimeAutotranslaterLocales::fromString($tLang)->eraFormatCeForSprintf(), $timeName);
        case NodaTimeAutotranslaterSuffixMode::BCE:
            return \sprintf(NodaTimeAutotranslaterLocales::fromString($tLang)->eraFormatBceForSprintf(), $timeName);
        default:
            throw new Exception("Unknown case encountered for time translations.");
        }

    }

    /**
     * Translated years or timespans below 1000 CE.
     *
     * @param array<integer|string> $timeInfo Time information.
     *
     * @return array<string, string>
     */
    public static function translateYearsWithSuffix(array $timeInfo):array {

        $start = \intval($timeInfo['zeit_beginn']);
        $end = \intval($timeInfo['zeit_ende']);

        $suffixMode = self::getSuffixModeForYearsWSuffix($start, $end);

        // Time info to pass
        if ($suffixMode === NodaTimeAutotranslaterSuffixMode::BCE) {
            $timeInfoToCopy = $timeInfo;
            $timeInfoToCopy["zeit_beginn"] = \abs($start);
            $timeInfoToCopy["zeit_ende"] = \abs($end);
        }

        $output = [];
        $cases = NodaTimeAutotranslaterLocales::cases();
        foreach ($cases as $tLang) {

            if ($suffixMode === NodaTimeAutotranslaterSuffixMode::BCE) {

                if (empty($timeInfoToCopy)) {
                    throw new Exception("No time information to copy existent");
                }
                $ceIndicatorsToRemove = explode("%s", $tLang->eraFormatCeForSprintf());
                $year = self::getTranslations($timeInfoToCopy)[$tLang->name];
                foreach ($ceIndicatorsToRemove as $ceIndicatorToRemove) {
                    $year = str_replace($ceIndicatorToRemove, "", $year);
                }
            }
            else {
                if ($start === $end) {
                    $year = \sprintf($tLang->formatSingleYearForSprintf(), (string)\abs($start));
                }
                else $year = \sprintf($tLang->formatYearspanForSprintf(), (string)\abs($start), (string)\abs($end));
            }

            $output[$tLang->name] = self::applyBcBceFormat($tLang->name, $year, $suffixMode);

        }
        return $output;

    }

    /**
     * Translated only years: 1994.
     *
     * @param array<integer|string> $timeInfo Time information.
     *
     * @return array<string, string>
     */
    public static function translateYearOnly(array $timeInfo):array {

        $start = \intval($timeInfo['zeit_beginn']);
        $output = [];

        $cases = NodaTimeAutotranslaterLocales::cases();
        foreach ($cases as $tLang) {
            $format = $tLang->formatSingleYearForSprintf();
            $output[$tLang->name] = \sprintf($format, (string)\abs($start));
        }
        return $output;

    }

    /**
     * Translated only years: 1994.
     *
     * @param array<integer|string> $timeInfo Time information.
     *
     * @return array<string, string>
     */
    public static function translateTimespanYears(array $timeInfo):array {

        $start = \intval($timeInfo['zeit_beginn']);
        $end = \intval($timeInfo['zeit_ende']);
        $output = [];

        $cases = NodaTimeAutotranslaterLocales::cases();
        foreach ($cases as $tLang) {
            $output[$tLang->name] = \sprintf($tLang->formatYearspanForSprintf(), (string)\abs($start), (string)\abs($end));
        }
        return $output;

    }

    /**
     * Translates century names: 19. Jahrhundert.
     *
     * @param array<integer|string> $timeInfo Time information.
     *
     * @return array<string, string>
     */
    public static function translateYearsAsCentury(array $timeInfo):array {

        if (\intval($timeInfo['zeit_beginn']) > 0) {
            // Beginn: 1500. (1501 - 1) / 100 + 1 = 16. 16th century is the time.
            $start_cen = ((\intval($timeInfo['zeit_beginn']) - 1) / 100) + 1;
            // End: 1600. 16th century is the time.
            $end_cen = \intval($timeInfo['zeit_ende']) / 100;
        }
        else {
            // End: -1600. 16th century is the time.
            $start_cen = \intval($timeInfo['zeit_beginn']) / 100;
            // End: -1500. (1501 - 1) / 100 + 1 = 16. 16th century is the time.
            $end_cen = ((\intval($timeInfo['zeit_ende']) + 1) / 100) - 1;
        }

        $suffixMode = self::getSuffixModeForYearsWSuffix(\intval($timeInfo['zeit_beginn']) - 1, \intval($timeInfo['zeit_ende']));

        $output = [];

        if ($start_cen === $end_cen) {
            foreach (self::LANGS_CENTURY_FORMAT as $tLang => $format) {
                $tLangValue = \sprintf($format, (string)\abs($start_cen));
                $output[$tLang] = self::applyBcBceFormat($tLang, $tLangValue, $suffixMode);
            }
        }
        else {
            foreach (self::LANGS_CENTURIES_FORMAT as $tLang => $format) {
                $tLangValue = \sprintf($format, (string)\abs($start_cen), (string)\abs($end_cen));
                $output[$tLang] = self::applyBcBceFormat($tLang, $tLangValue, $suffixMode);
            }
        }
        return $output;

    }

    /**
     * Translates decade names: 1920er Jahre.
     *
     * @param array<integer|string> $timeInfo Time information.
     *
     * @return array<string, string>
     */
    public static function translateYearsAsDecade(array $timeInfo):array {

        // Beginn: 1500. (1501 - 1) / 100 + 1 = 16. 16th century is the time.
        $start_cen = (\intval($timeInfo['zeit_beginn']));
        // End: 1600. 16th century is the time.
        $end_cen = (\intval($timeInfo['zeit_ende']));

        $suffixMode = self::getSuffixModeForYearsWSuffix(\intval($timeInfo['zeit_beginn']) - 1, \intval($timeInfo['zeit_ende']));

        $output = [];

        if ($start_cen === $end_cen - 9) {
            foreach (self::LANGS_DECADE_FORMAT as $tLang => $format) {
                $tLangValue = \sprintf($format, (string)$start_cen, (string)$end_cen);
                if (!empty(self::LANGS_SYLLABLE_CLEANING[$tLang])) $tLangValue = \strtr($tLangValue, self::LANGS_SYLLABLE_CLEANING[$tLang]);
                $output[$tLang] = self::applyBcBceFormat($tLang, $tLangValue, $suffixMode);
            }
        }
        else {
            foreach (self::LANGS_DECADES_FORMAT as $tLang => $format) {
                $tLangValue = \sprintf($format, (string)$start_cen, (string)($end_cen - 9));
                if (!empty(self::LANGS_SYLLABLE_CLEANING[$tLang])) $tLangValue = \strtr($tLangValue, self::LANGS_SYLLABLE_CLEANING[$tLang]);
                $output[$tLang] = self::applyBcBceFormat($tLang, $tLangValue, $suffixMode);
            }
        }
        return $output;

    }

    /**
     * Translated years or timespans below 1000 CE.
     *
     * @param array<integer|string> $timeInfo Time information.
     *
     * @return array<string, string>
     */
    public static function translateYearsAfterStart(array $timeInfo):array {

        if (empty(\trim((string)$timeInfo['zeit_zaehlzeit_monat'], " ,.;0"))) {
            $timeInfo['zeit_beginn'] = \strval(\intval($timeInfo['zeit_beginn']) - 1);
        }
        # else throw new Exception("Unhandled split case. Counting time is: " . var_export($timeInfo, true));
        $innerTimeInfo = $timeInfo;
        $innerTimeInfo['zeit_ende'] = $timeInfo['zeit_beginn'];
        $output = [];

        foreach (self::LANGS_AFTER_START_FORMAT_YEAR as $tLang => $format) {

            $dateAlone = self::getTranslations($innerTimeInfo)[$tLang];

            $timeName = \sprintf($format, $dateAlone);
            $output[$tLang] = $timeName;

        }
        return $output;

    }

    /**
     * Translated years or timespans below 1000 CE.
     *
     * @param array<integer|string> $timeInfo Time information.
     *
     * @return array<string, string>
     */
    public static function translateYearsSinceStart(array $timeInfo):array {

        $start = \intval($timeInfo['zeit_beginn']);

        if (substr((string)$timeInfo['zeit_name'], 0, 5) === "Nach ") {
            return self::translateYearsAfterStart($timeInfo);
        }

        $innerTimeInfo = $timeInfo;
        $innerTimeInfo['zeit_ende'] = $timeInfo['zeit_beginn'];
        $output = [];

        foreach (self::LANGS_SINCE_START_FORMAT_YEAR as $tLang => $format) {

            $dateAlone = self::getTranslations($innerTimeInfo)[$tLang];

            $timeName = \sprintf($format, $dateAlone);
            $output[$tLang] = $timeName;

        }
        return $output;

    }

    /**
     * Translated years or timespans below 1000 CE.
     *
     * @param array<integer|string> $timeInfo Time information.
     *
     * @return array<string, string>
     */
    public static function translateYearsBeforeEnd(array $timeInfo):array {

        if (empty(\trim((string)$timeInfo['zeit_zaehlzeit_monat'], " ,.;0"))) {
            $timeInfo['zeit_ende'] = \strval(\intval($timeInfo['zeit_ende']) + 1);
        }
        $end = \intval($timeInfo['zeit_ende']);

        $innerTimeInfo = $timeInfo;
        $innerTimeInfo['zeit_beginn'] = $timeInfo['zeit_ende'];

        $output = [];
        foreach (self::LANGS_BEFORE_START_FORMAT_YEAR as $tLang => $format) {

            $dateAlone = self::getTranslations($innerTimeInfo)[$tLang];

            $timeName = \sprintf($format, $dateAlone);
            $output[$tLang] = $timeName;
        }
        return $output;

    }

    /**
     * Translated years or timespans below 1000 CE.
     *
     * @param array<integer|string> $timeInfo Time information.
     *
     * @return array<string, string>
     */
    public static function translateYearsUntilEnd(array $timeInfo):array {

        $end = \intval($timeInfo['zeit_ende']);

        if (substr((string)$timeInfo['zeit_name'], 0, 4) === "Vor ") {
            return self::translateYearsBeforeEnd($timeInfo);
        }

        $innerTimeInfo = $timeInfo;
        $innerTimeInfo['zeit_beginn'] = $timeInfo['zeit_ende'];

        $output = [];
        foreach (self::LANGS_UNTIL_START_FORMAT_YEAR as $tLang => $format) {

            $dateAlone = self::getTranslations($innerTimeInfo)[$tLang];

            $timeName = \sprintf($format, $dateAlone);
            $output[$tLang] = $timeName;
        }
        return $output;

    }

    /**
     * Gets translations for a given entry type.
     *
     * @param array<integer|string> $timeInfo Time information.
     *
     * @return array<string>
     */
    public static function getTranslations(array $timeInfo):array {

        if (!empty($timeInfo['zeit_name']) and strlen((string)$timeInfo['zeit_name']) > 10 and !empty($timespanDates = NodaTimeSplitter::attempt_splitting_from_till((string)$timeInfo['zeit_name']))) {

            $output = [];

            $start = NodaTimeSplitter::attempt_splitting($timespanDates['start_name']);
            $startTimeInfo = [
                "zeit_name" => $timespanDates['start_name'],
                "zeit_beginn" => $start[0],
                "zeit_ende" => $start[1],
                "zeit_zaehlzeit_jahr" => NodaTimeSplitter::timePartsToCountingYear($start),
                "zeit_zaehlzeit_monat" => $start[2],
                "zeit_zaehlzeit_tag" => $start[3],
                "zeit_zaehlzeit_vorzeichen" => $start[4],
            ];

            $end = NodaTimeSplitter::attempt_splitting($timespanDates['end_name']);
            $endTimeInfo = [
                "zeit_name" => $timespanDates['end_name'],
                "zeit_beginn" => $end[0],
                "zeit_ende" => $end[1],
                "zeit_zaehlzeit_jahr" => NodaTimeSplitter::timePartsToCountingYear($end),
                "zeit_zaehlzeit_monat" => $end[2],
                "zeit_zaehlzeit_tag" => $end[3],
                "zeit_zaehlzeit_vorzeichen" => $end[4],
            ];

            $output = [];
            $cases = NodaTimeAutotranslaterLocales::cases();
            foreach ($cases as $tLang) {
                $start_term = self::getTranslations($startTimeInfo)[$tLang->name];
                $end_term = self::getTranslations($endTimeInfo)[$tLang->name];

                $output[$tLang->name] = \sprintf($tLang->formatYearspanForSprintf(), $start_term, $end_term);
            }
            return $output;
        }

        if (($translation_type = self::check_translatability((string)$timeInfo['zeit_beginn'], (string)$timeInfo['zeit_ende'], (string)$timeInfo['zeit_zaehlzeit_monat'])) === NodaTimeAutotranslaterStatus::NOT) {
            throw new MDgenericInvalidInputsException("Non-translatable date: {$timeInfo['zeit_beginn']} - {$timeInfo['zeit_ende']}");
        }

        if ($translation_type === NodaTimeAutotranslaterStatus::ONLY_YEAR) {
            return self::translateYearOnly($timeInfo);
        }

        if ($translation_type === NodaTimeAutotranslaterStatus::SINCE_START) {
            return self::translateYearsSinceStart($timeInfo);
        }

        if ($translation_type === NodaTimeAutotranslaterStatus::UNTIL_END) {
            return self::translateYearsUntilEnd($timeInfo);
        }

        if ($translation_type === NodaTimeAutotranslaterStatus::CENTURY) {
            return self::translateYearsAsCentury($timeInfo);
        }

        if ($translation_type === NodaTimeAutotranslaterStatus::DECADE) {
            return self::translateYearsAsDecade($timeInfo);
        }

        if ($translation_type === NodaTimeAutotranslaterStatus::TIMESPAN_YEARS) {
            return self::translateTimespanYears($timeInfo);
        }

        if ($translation_type === NodaTimeAutotranslaterStatus::AS_YEAR_WITH_SUFFIX) {
            return self::translateYearsWithSuffix($timeInfo);
        }

        $suffixMode = self::getSuffixModeForYearsWSuffix(\intval($timeInfo['zeit_beginn']) - 1, \intval($timeInfo['zeit_ende']));

        if (\trim((string)$timeInfo['zeit_zaehlzeit_tag'], ", .0") === "") {
            $dateStr = "{$timeInfo['zeit_zaehlzeit_jahr']}-{$timeInfo['zeit_zaehlzeit_monat']}-05 12:00:01";
            $usecase = NodaTimeAutotranslaterUseCase::MONTH;
        }
        else {
            $dateStr = "{$timeInfo['zeit_zaehlzeit_jahr']}-{$timeInfo['zeit_zaehlzeit_monat']}-{$timeInfo['zeit_zaehlzeit_tag']} 12:00:01";
            $usecase = NodaTimeAutotranslaterUseCase::DAY;
        }

        $dateGeneral = new DateTimeImmutable($dateStr);

        $output = [];
        $cases = NodaTimeAutotranslaterLocales::cases();
        foreach ($cases as $tLang) {

            $locale = $tLang->localeIntlDateFormatter();

            $dateGeneral = IntlCalendar::createInstance(locale: $locale);
            $dateGeneral->set(IntlCalendar::FIELD_YEAR, (int)$timeInfo['zeit_zaehlzeit_jahr']);
            $dateGeneral->set(IntlCalendar::FIELD_MONTH, (int)$timeInfo['zeit_zaehlzeit_monat'] - 1);
            if ($usecase === NodaTimeAutotranslaterUseCase::DAY) {
                $dateGeneral->set(IntlCalendar::FIELD_DAY_OF_MONTH, (int)$timeInfo['zeit_zaehlzeit_tag']);
            }

            $format = match($usecase) {
                NodaTimeAutotranslaterUseCase::MONTH => $tLang->formatYearMonthForIntlDateFormatter(),
                default => $tLang->formatFullDateForIntlDateFormatter(),
            };

            /*
            \setlocale(LC_TIME, $locale);
            $tLangValue = self::_formatTimeName($locale, $tLangValue, $dateGeneral);
            if ($locale !== \setlocale(LC_TIME, "0")) continue;
             */

            # \setlocale(LC_TIME, $locale);
            # if ($locale !== \setlocale(LC_TIME, "0")) continue;
            # $tLangValue = \strftime($format, $dateGeneral ?: 0);

            $fmt = new IntlDateFormatter(
                $locale,
                timezone: new DateTimeZone("GMT"),
                pattern: $format,
            );
            if (($tLangValue = $fmt->format($dateGeneral)) === false) continue;

            $output[$tLang->name] = self::applyBcBceFormat($tLang->name, $tLangValue, $suffixMode);

        }

        return $output;

    }

    /**
     * Runs autotranslater.
     *
     * @param array<integer|string> $timeInfo Time information.
     *
     * @return void
     */
    public function translate(array $timeInfo):void {

        $translations = self::getTranslations($timeInfo);

        $this->_mysqli_noda->autocommit(false);

        foreach ($translations as $tLang => $tLangValue) {
            $this->_insertStmt->bind_param("iss", $this->_znum, $tLang, $tLangValue);
            $this->_insertStmt->execute();
        }

        $this->_mysqli_noda->commit();
        $this->_mysqli_noda->autocommit(true);

    }

    /**
     * Constructor.
     *
     * @param MDMysqli $mysqli_noda Database connection.
     * @param integer  $znum        Time ID.
     *
     * @return void
     */
    public function __construct(MDMysqli $mysqli_noda, int $znum) {

        $this->_mysqli_noda = $mysqli_noda;
        $this->_znum = $znum;

        $this->_insertStmt = $this->_mysqli_noda->do_prepare("INSERT INTO `zeit_translation`
            (`zeit_id`, `trans_language`, `trans_name`)
            VALUES
            (?, ?, ?)");

    }

    /**
     * Destructor.
     *
     * @return void
     */
    public function __destruct() {

        $this->_insertStmt->close();

    }
}
